------------------------------------------------------------------------------
-- user_logic.vhd - entity/architecture pair
------------------------------------------------------------------------------
--
-- ***************************************************************************
-- ** Copyright (c) 1995-2011 Xilinx, Inc.  All rights reserved.            **
-- **                                                                       **
-- ** Xilinx, Inc.                                                          **
-- ** XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"         **
-- ** AS A COURTESY TO YOU, SOLELY FOR USE IN DEVELOPING PROGRAMS AND       **
-- ** SOLUTIONS FOR XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE,        **
-- ** OR INFORMATION AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE,        **
-- ** APPLICATION OR STANDARD, XILINX IS MAKING NO REPRESENTATION           **
-- ** THAT THIS IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,     **
-- ** AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE      **
-- ** FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY              **
-- ** WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE               **
-- ** IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR        **
-- ** REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF       **
-- ** INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS       **
-- ** FOR A PARTICULAR PURPOSE.                                             **
-- **                                                                       **
-- ***************************************************************************
--
------------------------------------------------------------------------------
-- Filename:          user_logic.vhd
-- Version:           1.00.a
-- Description:       User logic.
-- Date:              Mon Dec 26 18:49:34 2011 (by Create and Import Peripheral Wizard)
-- VHDL Standard:     VHDL'93
------------------------------------------------------------------------------
-- Naming Conventions:
--   active low signals:                    "*_n"
--   clock signals:                         "clk", "clk_div#", "clk_#x"
--   reset signals:                         "rst", "rst_n"
--   generics:                              "C_*"
--   user defined types:                    "*_TYPE"
--   state machine next state:              "*_ns"
--   state machine current state:           "*_cs"
--   combinatorial signals:                 "*_com"
--   pipelined or register delay signals:   "*_d#"
--   counter signals:                       "*cnt*"
--   clock enable signals:                  "*_ce"
--   internal version of output port:       "*_i"
--   device pins:                           "*_pin"
--   ports:                                 "- Names begin with Uppercase"
--   processes:                             "*_PROCESS"
--   component instantiations:              "<ENTITY_>I_<#|FUNC>"
------------------------------------------------------------------------------

-- DO NOT EDIT BELOW THIS LINE --------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

library proc_common_v3_00_a;
use proc_common_v3_00_a.proc_common_pkg.all;

-- DO NOT EDIT ABOVE THIS LINE --------------------

--USER libraries added here

------------------------------------------------------------------------------
-- Entity section
------------------------------------------------------------------------------
-- Definition of Generics:
--   C_SLV_AWIDTH                 -- Slave interface address bus width
--   C_SLV_DWIDTH                 -- Slave interface data bus width
--   C_NUM_MEM                    -- Number of memory spaces
--
-- Definition of Ports:
--   Bus2IP_Clk                   -- Bus to IP clock
--   Bus2IP_Resetn                -- Bus to IP reset
--   Bus2IP_Addr                  -- Bus to IP address bus
--   Bus2IP_CS                    -- Bus to IP chip select for user logic memory selection
--   Bus2IP_RNW                   -- Bus to IP read/not write
--   Bus2IP_Data                  -- Bus to IP data bus
--   Bus2IP_BE                    -- Bus to IP byte enables
--   Bus2IP_RdCE                  -- Bus to IP read chip enable
--   Bus2IP_WrCE                  -- Bus to IP write chip enable
--   Bus2IP_Burst                 -- Bus to IP burst-mode qualifier
--   Bus2IP_BurstLength           -- Bus to IP burst length
--   Bus2IP_RdReq                 -- Bus to IP read request
--   Bus2IP_WrReq                 -- Bus to IP write request
--   IP2Bus_AddrAck               -- IP to Bus address acknowledgement
--   IP2Bus_Data                  -- IP to Bus data bus
--   IP2Bus_RdAck                 -- IP to Bus read transfer acknowledgement
--   IP2Bus_WrAck                 -- IP to Bus write transfer acknowledgement
--   IP2Bus_Error                 -- IP to Bus error response
--   Type_of_xfer                 -- Transfer Type
------------------------------------------------------------------------------

entity user_logic is
  generic
  (
    -- ADD USER GENERICS BELOW THIS LINE ---------------
    --USER generics added here
    -- ADD USER GENERICS ABOVE THIS LINE ---------------

    -- DO NOT EDIT BELOW THIS LINE ---------------------
    -- Bus protocol parameters, do not add to or delete
    C_SLV_AWIDTH                   : integer              := 32;
    C_SLV_DWIDTH                   : integer              := 32;
    C_NUM_MEM                      : integer              := 4
    -- DO NOT EDIT ABOVE THIS LINE ---------------------
  );
  port
  (
    -- ADD USER PORTS BELOW THIS LINE ------------------
    --USER ports added here
	data_from_user0	               : in std_logic_vector(15 downto 0);
	data_from_user1	               : in std_logic_vector(15 downto 0);
	data_from_user2	               : in std_logic_vector(15 downto 0);
	data_from_user3	               : in std_logic_vector(15 downto 0);
	data_to_user	               : out std_logic_vector(15 downto 0);
	user_cs                        : out std_logic_vector(3 downto 0);
	user_rd                        : out std_logic;
	user_wr                        : out std_logic;
	user_rst                       : out std_logic;
	user_clk                       : out std_logic;
	user_add                       : out std_logic_vector(11 downto 0);
    -- ADD USER PORTS ABOVE THIS LINE ------------------

    -- DO NOT EDIT BELOW THIS LINE ---------------------
    -- Bus protocol ports, do not add to or delete
    Bus2IP_Clk                     : in  std_logic;
    Bus2IP_Resetn                  : in  std_logic;
    Bus2IP_Addr                    : in  std_logic_vector(C_SLV_AWIDTH-1 downto 0);
    Bus2IP_CS                      : in  std_logic_vector(C_NUM_MEM-1 downto 0);
    Bus2IP_RNW                     : in  std_logic;
    Bus2IP_Data                    : in  std_logic_vector(C_SLV_DWIDTH-1 downto 0);
    Bus2IP_BE                      : in  std_logic_vector(C_SLV_DWIDTH/8-1 downto 0);
    Bus2IP_RdCE                    : in  std_logic_vector(C_NUM_MEM-1 downto 0);
    Bus2IP_WrCE                    : in  std_logic_vector(C_NUM_MEM-1 downto 0);
    Bus2IP_Burst                   : in  std_logic;
    Bus2IP_BurstLength             : in  std_logic_vector(7 downto 0);
    Bus2IP_RdReq                   : in  std_logic;
    Bus2IP_WrReq                   : in  std_logic;
    IP2Bus_AddrAck                 : out std_logic;
    IP2Bus_Data                    : out std_logic_vector(C_SLV_DWIDTH-1 downto 0);
    IP2Bus_RdAck                   : out std_logic;
    IP2Bus_WrAck                   : out std_logic;
    IP2Bus_Error                   : out std_logic;
    Type_of_xfer                   : out std_logic
    -- DO NOT EDIT ABOVE THIS LINE ---------------------
  );

  attribute MAX_FANOUT : string;
  attribute SIGIS : string;

  attribute SIGIS of Bus2IP_Clk    : signal is "CLK";
  attribute SIGIS of Bus2IP_Resetn : signal is "RST";

end entity user_logic;

------------------------------------------------------------------------------
-- Architecture section
------------------------------------------------------------------------------

architecture IMP of user_logic is
	signal mem_select                     : std_logic_vector(3 downto 0);
	signal mem_read_enable                : std_logic;
	signal mem_write_enable               : std_logic;
	signal mem_read_req                   : std_logic;
	signal mem_write_req                   : std_logic;
	signal mem_read_enable_dly1           : std_logic;
	signal mem_write_enable_dly1           : std_logic;
	signal mem_ip2bus_data                : std_logic_vector(C_SLV_DWIDTH-1 downto 0);
	signal mem_read_ack_dly               : std_logic_vector(9 downto 0);
	signal mem_write_ack_dly               : std_logic_vector(9 downto 0);
	signal mem_read_ack                   : std_logic;
	signal mem_write_ack                  : std_logic;
begin
	user_cs         <= Bus2IP_CS;
	user_rd         <= mem_read_enable;
	user_wr         <= mem_write_enable; 
	user_clk        <= Bus2IP_Clk;
	user_rst        <= Bus2IP_Resetn;                   
	data_to_user    <= Bus2IP_Data(15 downto 0) when Bus2IP_BE(0) = '1' else Bus2IP_Data(31 downto 16);
	user_add        <= Bus2IP_Addr(12 downto 2) & Bus2IP_BE(0);

	mem_select      <= Bus2IP_CS;
	mem_read_enable <= ( Bus2IP_RdCE(0) or Bus2IP_RdCE(1) or Bus2IP_RdCE(2) or Bus2IP_RdCE(3) );
	mem_write_enable   <= ( Bus2IP_WrCE(0) or Bus2IP_WrCE(1) or Bus2IP_WrCE(2) or Bus2IP_WrCE(3) );
	mem_read_ack    <= mem_read_ack_dly(0);
	mem_write_ack   <= mem_write_ack_dly(0); 
	
	make_read_write_pulse : process( Bus2IP_Clk ) is
	begin
		if ( Bus2IP_Clk'event and Bus2IP_Clk = '1' ) then
			if ( Bus2IP_Resetn = '0' ) then
				mem_read_enable_dly1 <= '0';
				mem_write_enable_dly1 <= '0';
			else
				mem_read_enable_dly1 <= mem_read_enable;
				mem_write_enable_dly1 <= mem_write_enable;
			end if;
		end if;
	end process make_read_write_pulse;


	mem_read_req    <= mem_read_enable and not(mem_read_enable_dly1);
	mem_write_req   <= mem_write_enable and not(mem_write_enable_dly1);


	BRAM_RD_ACK_PROC : process( Bus2IP_Clk ) is
	begin

		if ( Bus2IP_Clk'event and Bus2IP_Clk = '1' ) then
			if ( Bus2IP_Resetn = '0' ) then
				mem_read_ack_dly <= (others => '0'); 
				mem_write_ack_dly <= (others => '0'); 
			else
				mem_read_ack_dly(9) <= mem_read_req;
				mem_read_ack_dly(8) <= mem_read_ack_dly(9);
				mem_read_ack_dly(7) <= mem_read_ack_dly(8);
				mem_read_ack_dly(6) <= mem_read_ack_dly(7);
				mem_read_ack_dly(5) <= mem_read_ack_dly(6);
				mem_read_ack_dly(4) <= mem_read_ack_dly(5);
				mem_read_ack_dly(3) <= mem_read_ack_dly(4);
				mem_read_ack_dly(2) <= mem_read_ack_dly(3);
				mem_read_ack_dly(1) <= mem_read_ack_dly(2);
				mem_read_ack_dly(0) <= mem_read_ack_dly(1);

				mem_write_ack_dly(9) <= mem_write_req;
				mem_write_ack_dly(8) <= mem_write_ack_dly(9);
				mem_write_ack_dly(7) <= mem_write_ack_dly(8);
				mem_write_ack_dly(6) <= mem_write_ack_dly(7);
				mem_write_ack_dly(5) <= mem_write_ack_dly(6);
				mem_write_ack_dly(4) <= mem_write_ack_dly(5);
				mem_write_ack_dly(3) <= mem_write_ack_dly(4);
				mem_write_ack_dly(2) <= mem_write_ack_dly(3);
				mem_write_ack_dly(1) <= mem_write_ack_dly(2);
				mem_write_ack_dly(0) <= mem_write_ack_dly(1);
			end if;
		end if;

	end process BRAM_RD_ACK_PROC;

  -- implement Block RAM read mux
	MEM_IP2BUS_DATA_PROC : process( data_from_user0, data_from_user1, data_from_user2, data_from_user3, mem_select ) is
	begin
		case mem_select is
			when "0001" => mem_ip2bus_data <= data_from_user0 & data_from_user0;
			when "0010" => mem_ip2bus_data <= data_from_user1 & data_from_user1;
			when "0100" => mem_ip2bus_data <= data_from_user2 & data_from_user2;
			when "1000" => mem_ip2bus_data <= data_from_user3 & data_from_user3;
			when others => mem_ip2bus_data <= (others => '0');
		end case;

	end process MEM_IP2BUS_DATA_PROC;

  ------------------------------------------
  -- Example code to drive IP to Bus signals
  ------------------------------------------
  IP2Bus_Data  <= mem_ip2bus_data when mem_read_ack = '1' else
                  (others => '0');

  IP2Bus_AddrAck <= (mem_write_enable and mem_write_ack) or (mem_read_enable and mem_read_ack);
  IP2Bus_WrAck <= mem_write_ack;
  IP2Bus_RdAck <= mem_read_ack;
  IP2Bus_Error <= '0';

end IMP;
